/* begin_generated_IBM_copyright_prolog                            */
/* 
 * Licensed Materials - Property of IBM 
 * 
 * 5724-N72 5655-R41 
 * 
 * Copyright IBM Corp. 2006, 2007 All Rights Reserved 
 * 
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with 
 * IBM Corp. 
 */
/* end_generated_IBM_copyright_prolog                            */
using System;
using System.Collections.Generic;
using System.Text;
using System.Security.Cryptography.X509Certificates;
using System.Net.Security;
using System.Net;
using System.Windows.Forms;
using System.ComponentModel;
using System.Data;
using System.Collections;
using System.Drawing;
using System.IO;
using System.Security.Cryptography;

/* This module performs the following functions:
 * 
 * - Creation of the Core Service and Ontology Service objects
 * - Addition of security credentials to both objects if required
 * - Encryption and Decryption of the password when stored to the persistent to Properties.Settings
 * - Acts as a central point for displaying Eception information
 */
namespace WSRRSample {
    class CommsHandler {
        private wsrrCoreAPI.WSRRCoreSDOService coreService;
        private WSRR_Ontology_Service ontologyService;
        private WSRR_Governance_Service governanceService;
        private String decryptedPassword;

        // IV and Key for encryption
        private static byte[] rijnIV = { 234, 12, 34, 83, 137, 198, 223, 76, 48, 42, 78, 101, 198, 211, 1, 93 };
        private static byte[] rijnKey = { 209, 64, 39, 183, 247, 57, 213, 36, 58, 63, 178, 201, 191, 13, 59, 88, 
                                         225, 34, 12, 67,  163, 39, 43,  67, 123, 3, 253, 13,  139, 202, 5, 217 };

        // The constructor sets the ServicePointManager to accept all SSL certificates and decrypts the stored password
        public CommsHandler() {
            // Accept all certificates
            System.Net.ServicePointManager.ServerCertificateValidationCallback = System.Net.ServicePointManager.ServerCertificateValidationCallback
                += delegate(object sender2, X509Certificate certificate, X509Chain chain, SslPolicyErrors sslPolicyErrors) {
                        return true;
                    };
            // Decrypt the password
            decryptedPassword = decryptPassword(Properties.Settings.Default.Password);

            // create our security credentials only if we have a server defined
            if (Properties.Settings.Default.Server != null && Properties.Settings.Default.Server != "")
                updateReferences();
        }

        // Creates the Core and Ontology services and adds the security credentials in necessary
        public void updateReferences() {
            try {
                // Create the core Service
                coreService = new wsrrCoreAPI.WSRRCoreSDOService();
                coreService.Url = Properties.Settings.Default.WSRRSample_wsrrCoreAPI_WSRRCoreSDOService;
            } catch (Exception e) {
                MessageBox.Show("Could not create the Core SDO Service, please check the settings in File / Preferences: " + e.Message);
                return;
            }

            try {
                // create the ontology Service
                ontologyService = new WSRR_Ontology_Service();
                ontologyService.Url = Properties.Settings.Default.WSRRSample_WSRR_Ontology_Service_WSRROntologyWSService;
            } catch (Exception e) {
                MessageBox.Show("Could not create the Ontology Service, please check the settings in File / Preferences: " + e.Message);
                return;
            }

            try
            {
                // create the governance Service
                governanceService = new WSRR_Governance_Service();
                governanceService.Url = Properties.Settings.Default.WSRRSample_WSRR_Governance_Service_WSRRGovernanceWSService;
            }
            catch (Exception e)
            {
                MessageBox.Show("Could not create the Governance Service, please check the settings in File / Preferences: " + e.Message);
                return;
            }

            if (Properties.Settings.Default.SecurityEnabled) {
                // Add credentials to core Service
                NetworkCredential netCredential = new NetworkCredential(Properties.Settings.Default.UserID, decryptedPassword);
                Uri uri = new Uri(coreService.Url);
                ICredentials credentials = netCredential.GetCredential(uri, "Basic");
                coreService.Credentials = credentials;

                // Add credentials to ontology Service
                netCredential = new NetworkCredential(Properties.Settings.Default.UserID, decryptedPassword);
                uri = new Uri(ontologyService.Url);
                credentials = netCredential.GetCredential(uri, "Basic");
                ontologyService.Credentials = credentials;

                // Add credentials to governance Service
                netCredential = new NetworkCredential(Properties.Settings.Default.UserID, decryptedPassword);
                uri = new Uri(governanceService.Url);
                credentials = netCredential.GetCredential(uri, "Basic");
                governanceService.Credentials = credentials;

                // flag the services to send the authentication
                coreService.PreAuthenticate = true;
                ontologyService.PreAuthenticate = true;
                governanceService.PreAuthenticate = true;
            }

        }

        public wsrrCoreAPI.WSRRCoreSDOService getCoreService() {
            return coreService;
        }

        public WSRR_Ontology_Service getOntologyService() {
            return ontologyService;
        }

        public WSRR_Governance_Service getGovernanceService()
        {
            return governanceService;
        }

        public void HandleException(String msg, Exception e) {
            MessageBox.Show(msg + ": " + e.Message,"Error");
        }

        // Encrypt the provided password using RijndaelManaged encryption
        public String encryptPassword(String s) {
            decryptedPassword = s;

            if (s.Length > 0) {
                // Set length to 4x the source to ensure enough room for base64 encoding
                MemoryStream ms = new MemoryStream(s.Length * 4);
                SymmetricAlgorithm rijn = SymmetricAlgorithm.Create(); //Creates the default implementation, which is RijndaelManaged.         
                CryptoStream encStream = new CryptoStream(ms, rijn.CreateEncryptor(rijnKey, rijnIV), CryptoStreamMode.Write);
                encStream.Write(Encoding.ASCII.GetBytes(s), 0, s.Length);
                encStream.Close();

                return Convert.ToBase64String(ms.ToArray());
            } else
                return "";
        }

        // Decrypt the provided password using RijndaelManaged decryption
        public String decryptPassword(String s) {
            if (s.Length > 0) {
                byte[] nonBase64 = Convert.FromBase64String(s);

                // descrypted String will be shorted than the source so just set the length to be the same
                MemoryStream ms = new MemoryStream(s.Length);
                SymmetricAlgorithm rijn = SymmetricAlgorithm.Create(); //Creates the default implementation, which is RijndaelManaged.         
                CryptoStream decStream = new CryptoStream(ms, rijn.CreateDecryptor(rijnKey, rijnIV), CryptoStreamMode.Write);
                decStream.Write(nonBase64, 0, nonBase64.Length);
                decStream.Close();

                return Encoding.ASCII.GetString(ms.ToArray());
            } else
                return "";
        }

        public String getDecryptedPassword() {
            return decryptedPassword;
        }

    }
}
