/* begin_generated_IBM_copyright_prolog                            */
/* 
 * Licensed Materials - Property of IBM 
 * 
 * 5724-N72 5655-R41 
 * 
 * Copyright IBM Corp. 2006, 2007 All Rights Reserved 
 * 
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with 
 * IBM Corp. 
 */
/* end_generated_IBM_copyright_prolog                            */
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.IO;
using System.Text;
using System.Windows.Forms;
using WSRRSample.wsrrCoreAPI;

namespace WSRRSample {
    public partial class CreateCollectionForm : Form {
        WSRRCoreSDOService coreService;

        // List of all the files that will be added to the collection
        List<string> fullFileNames = new List<string>();

        // Constructor
        public CreateCollectionForm() {
            InitializeComponent();
            coreService = Program.comms.getCoreService();
            groupBox2_Resize(null,null);

            fileNameMenuStrip.Enabled = false;
        }

        // Called when the user clicks the "Create" button.
        private void createButton_Click(object sender, EventArgs e) {
            this.Cursor = Cursors.WaitCursor;

            // If the user has not provided a name then use "defaultCollection"
            string collectionName = collectionNameTextBox.Text;
            if ((collectionName == null) || (collectionName == "")) {
                collectionName = "defaultCollection";
            }

            // Create a GenericObject and assign it the values the user has provided
            GenericObject collectionGO = new GenericObject();
            collectionGO.bsrURI = "_1";
            collectionGO.name = collectionName;
            collectionGO.description = collectionDescriptionTextBox.Text;
            collectionGO.@namespace = collectionNamespaceTextBox.Text;
            collectionGO.version = collectionVersionTextBox.Text;

            // Now create a array of Documents from the files the user has specified
            List<Document> sourceDocuments = new List<Document>();
            String addedbsrURIs = "";

            try {

                // Loop over the list of files the user has provided and create an SDO object of the relevant type
                int count = 2;
                foreach (ListViewItem item in fileNamesListView.Items) {

                    // Get the full file name
                    string sourceFileName = fullFileNames.ToArray()[item.Index];
                    byte[] content = null;

                    // Load the file
                    FileStream inputFileStream = new FileStream(sourceFileName, FileMode.Open, FileAccess.Read, FileShare.Read);
                    long fileSize = inputFileStream.Length;
                    if (fileSize < Int32.MaxValue) {
                        content = new byte[fileSize];
                        inputFileStream.Read(content, 0, (int)fileSize);
                    } else {
                        MessageBox.Show("File size is too big");
                        this.Cursor = Cursors.WaitCursor;
                        return;
                    }

                    // Create an SDO object from the file
                    Document sourceDocument = Utilities.createCoreAPIDocOfType(item.SubItems[1].Text);

                    // Assign the document  a temporary bsrURI, we start at _2 and then count up
                    sourceDocument.bsrURI = "_" + count++;

                    // Keep a list of all the temporary bsrURIs we have created so that we can set them in our GenericObject
                    addedbsrURIs += sourceDocument.bsrURI + " ";

                    // Set content and location
                    sourceDocument.content = content;
                    sourceDocument.location = sourceFileName;

                    // Just hard-code name here to be the base name
                    sourceDocument.name = sourceFileName.Substring(sourceFileName.LastIndexOf("\\") + 1);

                    // Add the Document to the array
                    sourceDocuments.Add(sourceDocument);
                }

                // Create the relationship between the collection and the documents.
                UserDefinedRelationship udr = new UserDefinedRelationship();
                udr.name = "collectionRelationship";
                
                //Add the targets, but remove any trailing spaces
                udr.targets = addedbsrURIs.TrimEnd();                

                UserDefinedRelationship[] udrs = new UserDefinedRelationship[1];
                udrs[0] = udr;
                collectionGO.userDefinedRelationships = udrs;

                // Now build the create request containing the GO and the documents
                create createRequest = new create();
                createRequest.datagraph = new DataGraphType();
                createRequest.datagraph.WSRR = new WSRR();
                createRequest.datagraph.WSRR.artefacts = new BaseObject[sourceDocuments.Count + 1];
                createRequest.datagraph.WSRR.artefacts[0] = collectionGO;

                // Copy the Document objects we created as we loaded each file into the
                // artefacts array of the datagraph
                count = 1;
                foreach (Document doc in sourceDocuments) {
                    createRequest.datagraph.WSRR.artefacts[count++] = doc;
                }

                // Specify that _1 (the GenericObject that represents our collection) is our root object
                createRequest.datagraph.WSRR.root = "_1";

                // Just copy across the current udps from the form           
                UserDefinedProperty[] udps = GUI_Utilities.generateUDP(userPropertiesDataGridView);

                collectionGO.userDefinedProperties = udps;

                createResponse createResponse = coreService.create(createRequest);
            } catch (Exception we) {
                Program.comms.HandleException("Could not create the collection: ", we);
                return;
            } finally {
                this.Cursor = Cursors.Default;
            }

            MessageBox.Show("Collection created successfully");
            this.Close();
        }

        // Called when the user clicks the "Add Files" button
        private void addFilesButton_Click(object sender, EventArgs e) {

            // Show the open file dialog and then add all selected file names to the fullFileNames list
            // and the relative basename of each file, along with a default type, to the fileNamesListView
            if (openFileDialog1.ShowDialog() == DialogResult.OK) {

                foreach (string file in openFileDialog1.FileNames) {

                    //
                    String fType = Utilities.getFileType(file);

                    // Add a blank filename for now, as we'll update it to be relative in a moment
                    ListViewItem item = new ListViewItem("");
                    item.SubItems.Add(fType);
                    fileNamesListView.Items.Add(item);

                    // Put the full path into here so we can generate the relative paths in a moment
                    fullFileNames.Add(file);

                }
            }

            // Here we go through all teh file names that have currently been loaded and then work
            // out how much of each file's full path name we need to include in the ListView to
            // make clear the relative positions of the files on the file system.

            // Work out the matching portions of the paths
            String matchingPath = findMatchingPaths();

            // Change the displayed paths to be relative
            if (matchingPath != null && matchingPath.Length > 0) {
                int count = 0;
                foreach (ListViewItem item in fileNamesListView.Items) {
                    item.Text = fullFileNames.ToArray()[count++].Remove(0, matchingPath.Length);
                }
            }

        }

        // From the list of full file names in fullFileNames find the longest
        // String that matches the beginning of all the files.
        //
        // i.e: given the files: C:\a\b\y.jk and C:\a\c\x.jk
        //      this function will return C:\a\
        private String findMatchingPaths() {
            String matching = null;

            foreach (String path in fullFileNames) {
                if (matching == null) {
                    // Always remove the filename
                    matching = path.Substring(0,path.LastIndexOf("\\") + 1);
                } else {
                    int count = 0;
                    bool matches = true;
                    int lastSlash = -1;
                    while (count < matching.Length && count < path.Length && matches) {
                        if (matching[count] != path[count])
                            matches = false;
                        else {
                            if (matching[count] == '\\')
                                lastSlash = count;

                            count++;
                        }
                    }

                    // Only match upto the last backslash
                    if (lastSlash > 0)
                        matching = matching.Substring(0, lastSlash + 1);
                    else
                        matching = "";
                }
            }

            return matching;
        }

        // Loop through all the selected entries in the collection contents list and apply
        // the user's selected file type to them
        private void setFileTypes(string fType) {
            foreach (ListViewItem item in fileNamesListView.SelectedItems) {
                item.SubItems.RemoveAt(1);
                item.SubItems.Add(fType); ;
            }
        }

        // Called when the user selects a file type from the context menu
        private void toolStripMenuItemTypes_Click(object sender, EventArgs e) {
            // Change all selected docs to the sender's type
            setFileTypes(((ToolStripMenuItem)sender).Text);
        }

        // Called when the user selects Remove from the context menu, or when they
        // press the Remove button
        private void removeToolStripMenuItem_Click(object sender, EventArgs e) {
            foreach (ListViewItem item in fileNamesListView.SelectedItems) {
                fullFileNames.RemoveAt(item.Index);
                fileNamesListView.Items.Remove(item);
            }
        }

        // Called when the user changes the selected items in the collection contents list.
        // If there are any files selected in the list then we enable the buttons and the context menu
        private void fileNamesListView_SelectedIndexChanged(object sender, EventArgs e) {
            if (fileNamesListView.SelectedItems.Count > 0) {
                removeButton.Enabled = true;
                changeTypeComboBox.Enabled = true;
                fileNameMenuStrip.Enabled = true;
            }  else {
                removeButton.Enabled = false;
                changeTypeComboBox.Enabled = false;
                changeTypeComboBox.SelectedItem = null;
                fileNameMenuStrip.Enabled = false;
            }
        }

        // Closes the window
        private void cancelButton_Click(object sender, EventArgs e) {
            this.Close();
        }

        // Called when the user selects an entry from the Change type combo box
        private void changeTypeComboBox_SelectionChangeCommitted(object sender, EventArgs e) {
            setFileTypes(changeTypeComboBox.SelectedItem.ToString());
        }

        // Ensure the user properties DataGridView columns resize with the form
        private void userPropertiesDataGridView_Resize(object sender, EventArgs e)
        {
            userPropertiesDataGridView.Columns[1].Width = userPropertiesDataGridView.Width - userPropertiesDataGridView.Columns[0].Width;
        }

        // Ensure the Collection Contents List columns resize with the form
        private void fileNamesListView_Resize(object sender, EventArgs e)
        {
            fileNamesListView.Columns[0].Width = fileNamesListView.Width - fileNamesListView.Columns[1].Width - 20;
        }

        // Ensure the modeled properties resize with the form
        private void groupBox2_Resize(object sender, EventArgs e)
        {
            collectionNameTextBox.Width = (groupBox2.Width - collectionNameTextBox.Left - 10) / 2;
            collectionVersionTextBox.Width = (groupBox2.Width - collectionVersionTextBox.Left - 10) / 2;
            collectionNamespaceTextBox.Width = (groupBox2.Width - collectionNamespaceTextBox.Left - 10) / 2;
        }
    }
}