/* begin_generated_IBM_copyright_prolog                            */
/* 
 * Licensed Materials - Property of IBM 
 * 
 * 5724-N72 5655-R41 
 * 
 * Copyright IBM Corp. 2006, 2007 All Rights Reserved 
 * 
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with 
 * IBM Corp. 
 */
/* end_generated_IBM_copyright_prolog                            */
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.IO;
using System.Text;
using System.Windows.Forms;
using WSRRSample.wsrrCoreAPI;

namespace WSRRSample {
    public partial class CreateSingleDocumentForm : Form {
        WSRRCoreSDOService coreService;

        // Constructor
        public CreateSingleDocumentForm() {
            InitializeComponent();
            coreService = Program.comms.getCoreService();
            groupBox3_Resize(null, null);
        }

        // Called when the user clicks "Browse". Open the system fiel dialog and allow
        // the user to select a file to load into WSRR
        private void browseButton_Click(object sender, EventArgs e) {
            if (openFileDialog1.ShowDialog() == DialogResult.OK) {                
                fileNameTextBox.Text = openFileDialog1.FileName;

                // Only auto-set the file type if the user has not already selected 
                // one from teh combo box
                if (changeTypeComboBox.Text == null || changeTypeComboBox.Text == "" )
                    changeTypeComboBox.Text = Utilities.getFileType(fileNameTextBox.Text);
            }
        }

        // Called when the user clicks the "Create" button.
        // Load the specified file from the filesystem and create a Document to load into WSRR
        private void createButton_Click(object sender, EventArgs e) {

            this.Cursor = Cursors.WaitCursor;

            // Ask teh user to select a file
            string fileName = openFileDialog1.FileName;
            byte[] content = null;

            try {
                FileStream inputFileStream = new FileStream(fileName, FileMode.Open, FileAccess.Read, FileShare.Read);
                long fileSize = inputFileStream.Length;
                if (fileSize < Int32.MaxValue) {
                    content = new byte[fileSize];
                    inputFileStream.Read(content, 0, (int)fileSize);
                } else {
                    MessageBox.Show("File size is too big");
                    this.Cursor = Cursors.Default;
                    return;
                }

                Document doc = null;

                if (changeTypeComboBox.Text != null || changeTypeComboBox.Text == "")
                    doc = Utilities.createCoreAPIDocOfType(changeTypeComboBox.Text);
                else
                    doc = Utilities.createCoreAPIDocOfType(changeTypeComboBox.SelectedItem.ToString());

                // Set the temporary bsrURI
                doc.bsrURI = "_1";

                // Set the document content to be the file we loaded
                doc.content = content;

                doc.location = fileName;
                // Need to set name etc.
                if (docNameTextBox.Text != "") {
                    doc.name = docNameTextBox.Text;
                } else {
                    doc.name = fileName.Substring(fileName.LastIndexOf("\\") + 1);
                }
                // The namespace should not be set for some document types
                if (docNamespaceTextBox.Text != "") {
                    doc.@namespace = docNamespaceTextBox.Text;
                }
                // The version should not be set for some document types
                if (docVersionTextBox.Text != "") {
                    doc.version = docVersionTextBox.Text;
                }
                // Set the description
                if (docDescriptionTextBox.Text != "") {
                    doc.description = docDescriptionTextBox.Text;
                }

                doc.userDefinedProperties = GUI_Utilities.generateUDP(userPropertiesDataGridView);

                create createRequest = new create();
                createRequest.datagraph = new DataGraphType();
                createRequest.datagraph.WSRR = new WSRR();
                createRequest.datagraph.WSRR.artefacts = new BaseObject[1];
                createRequest.datagraph.WSRR.artefacts[0] = doc;
                createRequest.datagraph.WSRR.root = "_1";

                createResponse createResponse = coreService.create(createRequest);
            } catch (Exception we) {
                Program.comms.HandleException("Could not create the document: ", we);
                return;
            } finally {
                this.Cursor = Cursors.Default;
            }

            MessageBox.Show("Document created successfully");
            this.Close();

        }

        // Close the form
        private void cancelButton_Click(object sender, EventArgs e) {
            this.Close();
        }

        // Ensure the user properties DataGridView columns resize with the form
        private void userPropertiesDataGridView_Resize(object sender, EventArgs e)
        {
            userPropertiesDataGridView.Columns[1].Width = userPropertiesDataGridView.Width - userPropertiesDataGridView.Columns[0].Width;
        }

        // Ensure the modeled properties resize with the form
        private void groupBox3_Resize(object sender, EventArgs e)
        {            
            docNameTextBox.Width = (groupBox3.Width - docNameTextBox.Left - 10) / 2;
            docVersionTextBox.Width = (groupBox3.Width - docVersionTextBox.Left - 10) / 2;
            docNamespaceTextBox.Width = (groupBox3.Width - docNamespaceTextBox.Left - 10) / 2;
        }
    }
}