/* begin_generated_IBM_copyright_prolog                            */
/* 
* Licensed Materials - Property of IBM 
* 
* 5724-N72 5655-R41 
* 
* Copyright IBM Corp. 2006, 2007 All Rights Reserved 
* 
* US Government Users Restricted Rights - Use, duplication or 
* disclosure restricted by GSA ADP Schedule Contract with 
* IBM Corp. 
*/
/* end_generated_IBM_copyright_prolog                            */
using System;
using System.Text;
using System.Data;
using System.Collections.Generic;
using WSRRSample.wsrrCoreAPI;
using System.Windows.Forms;


/* This module performs the following GUI related functions:
 * 
 * - Add the desired columns to a DataTable object
 * - Populate a DataTable object from a DataGraphType object
 * - Generate a UserDefinedProperty[] to use in a call to WSRR from a DataGridView control
 */
namespace WSRRSample
{
    class GUI_Utilities
    {
        // Add the columns for the properties we want to display. Return the index of the bsrURI column
        // so that the caller can easily find the bsrURI of the object to use in other calls to WSRR.
        public static int addColumns(DataTable queryResultsDataTable)
        {
            // First column is name
            DataColumn nameColumn = new DataColumn("name", typeof(string));
            queryResultsDataTable.Columns.Add(nameColumn);

            // Second column is namespace
            DataColumn namespaceColumn = new DataColumn("namespace", typeof(string));
            queryResultsDataTable.Columns.Add(namespaceColumn);

            // Third column is version
            DataColumn versionColumn = new DataColumn("version", typeof(string));
            queryResultsDataTable.Columns.Add(versionColumn);


            // Fourth is description
            DataColumn descriptionColumn = new DataColumn("description", typeof(string));
            queryResultsDataTable.Columns.Add(descriptionColumn);

            // Fifth is bsrURI
            DataColumn bsrURIColumn = new DataColumn("bsrURI", typeof(string));
            queryResultsDataTable.Columns.Add(bsrURIColumn);

            // bsrURI is special so return it's index for use by the caller. Zero based so column 5 is index 4.
            return 4;
        }

        // Copy the desired properties from each object in the results array to a row in the DataTable
        public static void fillDataTable(DataTable resultsDataTable, DataGraphType[] results)
        {
            // Copy the column names from the DataTable into an array so we can copy the correct
            // properties out of out BaseObjects and into the DataTable
            int numberOfColumnsRequired = resultsDataTable.Columns.Count;
            DataColumn[] dataColumns = new DataColumn[numberOfColumnsRequired];
            resultsDataTable.Columns.CopyTo(dataColumns, 0);
            string[] propertyNamesRequired = new string[numberOfColumnsRequired];
            for (int i = 0; i < numberOfColumnsRequired; i++)
            {
                propertyNamesRequired[i] = dataColumns[i].ColumnName;
            }

            
            // Get the root object of each result set and copy it's properties to the Datatable
            for (int i = 0; i < results.Length; i++)
            {
                wsrrCoreAPI.DataGraphType result = results[i];
                if (result != null)
                {
                    wsrrCoreAPI.BaseObject baseObject = Utilities.getRootObject(result);
                    if (baseObject != null)
                    {
                        String[] values = new String[numberOfColumnsRequired];
                        for (int j = 0; j < numberOfColumnsRequired; j++)
                        {
                            switch (propertyNamesRequired[j])
                            {
                                case "name":
                                    values[0] = baseObject.name;
                                    break;

                                case "namespace":
                                    values[1] = baseObject.@namespace;
                                    break;

                                case "version":
                                    values[2] = baseObject.version;
                                    break;

                                case "description":
                                    values[3] = baseObject.description;
                                    break;

                                case "bsrURI":
                                    values[4] = baseObject.bsrURI;
                                    break;

                                default:
                                    // A property we don't want to display right now
                                    break;
                            }
                        }
                        resultsDataTable.Rows.Add(values);
                    }
                    else
                    {
                        // Property Query case
                        wsrrCoreAPI.UserDefinedProperty[] udps = result.PropertyQueryResult;
                        String[] values = new String[numberOfColumnsRequired];

                        //Copy the properties requested to the DataTable
                        for (int j = 0; j < numberOfColumnsRequired; j++)
                        {

                            for (int k = 0; k < udps.Length; k++)
                            {
                                wsrrCoreAPI.UserDefinedProperty udp = udps[k];
                                if (udp.name == propertyNamesRequired[j])
                                {
                                    values[k] = udp.value;
                                }
                            }
                        }
                        resultsDataTable.Rows.Add(values);
                    }
                }
            }
        }

        /* Generates a UserDefinedProperty[] to use in a call to WSRR from a DataGridView control */
        public static UserDefinedProperty[] generateUDP(DataGridView userPropertiesDataGridView)
        {
            // Just copy across the current udps from the form            
            wsrrCoreAPI.UserDefinedProperty[] udps = null;
            if (userPropertiesDataGridView.Rows.Count > 0)
            {

                //Check for any rows with a null property name
                int blanks = 0;
                foreach (DataGridViewRow row in userPropertiesDataGridView.Rows)
                {
                    // Index 0 is the property name
                    if (((String)row.Cells[0].Value) == null || ((String)row.Cells[0].Value) == "")
                        blanks++;
                }

                // Copy all user properties across apart from any with a blank property name
                udps = new WSRRSample.wsrrCoreAPI.UserDefinedProperty[userPropertiesDataGridView.Rows.Count - blanks];
                int index = 0;
                foreach (DataGridViewRow row in userPropertiesDataGridView.Rows)
                {
                    if (((String)row.Cells[0].Value) != null && ((String)row.Cells[0].Value) != "")
                    {
                        wsrrCoreAPI.UserDefinedProperty udp = new WSRRSample.wsrrCoreAPI.UserDefinedProperty();
                        udp.name = (String)row.Cells[0].Value;
                        udp.value = (String)row.Cells[1].Value;
                        udps[index] = udp;
                        index++;
                    }
                }
            }

            return udps;
        }

        // Builds a list of all TreeNodes that have Checked==true
        private static List<String> nodesChecked(TreeNodeCollection nodes)
        {
            List<String> ids = new List<String>();

            foreach (TreeNode node in nodes)
            {
                if (node.Checked)
                {
                    ids.Add((String)node.Tag);
                }

                ids.AddRange(nodesChecked(node.Nodes));
            }

            return ids;
        }

        // Uses nodesChecked() to build a list of all Checked nodes in treeView
        public static String[] getTreeViewSelectedUris(TreeView treeView) {

            List<String> ids = new List<String>();
            
            TreeNodeCollection nodes = treeView.Nodes;

            ids = nodesChecked(nodes);

            if (ids.Count > 0)
            {
                return ids.ToArray();
            }
            else
                return null;
        }

        // Sets the Chacked state of every TreeNode in treeView that has a Tag value that matches
        // any of the Strings in classifications.
        public static void setCheckBoxes(TreeView treeView, String[] classifications)
        {
            for (int c = 0; c < classifications.Length; c++)
            {
                TreeNode tn = findTreeNodeWithTag(treeView.Nodes,classifications[c]);
                
                if (tn != null)
                {
                    tn.Checked = true;
                    expandParents(tn);
                }
            }
        }

        // Returns the TreeNode that has a Tag attribute that matches tag
        public static TreeNode findTreeNodeWithTag(TreeNodeCollection nodes, String tag)
        {
            TreeNode found = null;

            int c = -1;
            while(++c < nodes.Count && found == null)
            {
                TreeNode node = nodes[c];
                
                if (tag.Trim() == (String)node.Tag)
                {
                    return node;
                }

                found = findTreeNodeWithTag(node.Nodes,tag);
            }

            return found;
        }

        // Returns the TreeNode that has a Tag attribute that matches tag
        public static TreeNode findTreeNodeWithTag(TreeView treeView, String tag)
        {
            TreeNodeCollection nodes = treeView.Nodes;

            return findTreeNodeWithTag(nodes,tag);

        }
        
        // This function steps back up through the tree all the way to the top-level Parent
        // calling the Expand() function on each node as it goes. This allows us to only
        // expand the parts of a TreeView we are interested in
        public static void expandParents(TreeNode node)
        {
            TreeNode parent = node.Parent;
            TreeNode nextParent = null;
            
            if (parent != null)
                nextParent = parent.Parent;

            while (nextParent != null) {
                parent.Expand();
                parent = nextParent;
                nextParent = nextParent.Parent;
            }

            if (parent != null)
                parent.Expand();
        }

        // Unchecks all nodes in the TreeNodeCollection
        private static void resetCheckBoxes(TreeNodeCollection nodes)
        {
            foreach (TreeNode node in nodes)
            {
                node.Checked = false;
                resetCheckBoxes(node.Nodes);
            }
        }

        // Unchecks all nodes in the TreeView
        public static void resetCheckBoxes(TreeView treeView)
        {
            resetCheckBoxes(treeView.Nodes);
        }

        // Visits all nodes in treeView and call the Close() function to copy them
        // into the nodes parameter.
        public static void cloneTreeViewNodes(TreeNodeCollection nodes, TreeView treeView)
        {
            // Clear out the destination node collection
            nodes.Clear();

            // We only need to visit each top-level member as the Clone() function
            // will copy all lower level members of each node
            foreach (TreeNode node in treeView.Nodes)
            {
                nodes.Add((TreeNode)node.Clone());
            }
        }

    }
}
