/* begin_generated_IBM_copyright_prolog                            */
/* 
 * Licensed Materials - Property of IBM 
 * 
 * 5724-N72 5655-R41 
 * 
 * Copyright IBM Corp. 2006, 2007 All Rights Reserved 
 * 
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with 
 * IBM Corp. 
 */
/* end_generated_IBM_copyright_prolog                            */
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Collections;
using System.Text;
using System.Threading;
using System.Windows.Forms;
using WSRRSample.wsrrCoreAPI;

/* This module is the Main Window of the sample application. 
 * 
 * It has 4 dialog windows:
 * 
 * - Preferences: This allows you to set the server host name, port and any security credentials
 * - Create a Generic Object: Allows you to create a generic object with modelled and user defined properties
 * - Create a Single Document: Allows you to create a single document from a file on the local machine
 * - Create a Collection: Allows you to create a collection from multiple files on the local machine
 * 
 * It has 2 tabs:
 *
 * - Query: This allows you to run an xPath style query against a WSRR instance
 * - Ontology: This allows you to browse the ontologies present in a WSRR instance.
 */
namespace WSRRSample
{
    public partial class MainForm : Form
    {
        // Helper class that stores a Label and an Id so we can save an extra parameter with each
        // entry in a list. This is used to store the governance uri with the name in a listbox.
        public class LabelId
        {
            public String label;
            public String id;

            public LabelId(String _label, String _id)
            {
                label = _label;
                id = _id;
            }

            // Return the label as the ToString() implementation
            public override String ToString()
            {
                return label;
            }
        }


        // Used to control the initialization of the 3 services after
        // the preferences dialog box has been displayed
        bool applicationLoaded = false;

        // Used to stop the xPath entry box and the classifications tree view from
        // interfering with each other. This is needed as the changes in one box are
        // reflected in the other.
        bool loadingQuery = false;

        // The WSRR 3 service objects
        WSRRCoreSDOService coreService = null;
        WSRR_Ontology_Service ontologyService = null;
        WSRR_Governance_Service governanceService = null;

        // The currently retrieved object from WSRR
        BaseObject retrievedObject = null;

        // The governance record for retrievedObject
        GovernanceRecord gR = null;        

        // The index of the column containing the BsrURI. We use this to allow
        // the results grid view to delete or retrieve an entity.
        int bsrURIColumnIndex = -1;
       
        /*
         * Constructs the MainForm and initializes the components within:
         * 
         * - If this is the first time the program has been run (Server == null)
         *   then show the preferences dialog, otherwise show the splash screen.
         * - Load the saved settings into the Preferences form.
         * - Initialize various variables.
         * - Load the queries and ontologies from WSRR.
         */
        public MainForm()
        {

            InitializeComponent();
            Thread th = null;

            // Unless this is the first time the app has been run then show the splash screen 
            // as we can take a while to load up the ontologies and saved queries
            if (Properties.Settings.Default.Server != null && Properties.Settings.Default.Server.Length > 0)
            {
                th = new Thread(new ThreadStart(DoSplash));
                th.Start();                                
            }

            // Create a new comms handler class that will be used by the rest of the code
            // to get references to the Core and Ontology services.         
            Program.comms = new CommsHandler();

            // If the server has not been setup then open the preferences form
            if (Properties.Settings.Default.Server == null || Properties.Settings.Default.Server.Length == 0) {
                MessageBox.Show(this, "As this is the first time you have run this application the preferences dialog will now display to allow you to setup the server details","WSRR Sample Client");
                preferencesToolStripMenuItem_Click(null, null);
                
                th = new Thread(new ThreadStart(DoSplash));
                th.Start();

                Program.comms.updateReferences();
            }

            coreService = Program.comms.getCoreService();
            ontologyService = Program.comms.getOntologyService();
            governanceService = Program.comms.getGovernanceService();

            // Setup the query tab
            queryDepthComboBox.SelectedIndex = 0;            
            queryResultsContextMenuStrip.Enabled = false;

            // Setup the retrieve tab
            retrieveDepthComboBox.SelectedIndex = 0;
            fileTypeLabel.Text = "";
            bsrURILabel.Text = "";
            docGovernanceStateLabel.Text = "";
            trueRadioButton.Select();
          
            // load queries and ontologies
            getRegistryInfo();

            // Force the control to call all the resize functions to ensure
            // the initial layout is correct
            this.Width = this.Width + 1;
            this.Width = this.Width;

            // Close the splash screen
            if (th != null)
                th.Abort();

            // Flag that we have finished initialization
            applicationLoaded = true;
            
        }

        // Show the Splash screen
        private void DoSplash()
        {
            Splash sp = new Splash();
            sp.ShowDialog();
        }

        // Show the preferences dialog. If the user clicks OK and the app has finished initialization
        // then request that the CommsHandler initializes the services.
        private void preferencesToolStripMenuItem_Click(object sender, EventArgs e)
        {
            this.Cursor = Cursors.WaitCursor;
            PreferencesForm form = new PreferencesForm();
            DialogResult RC = form.ShowDialog();

            if (RC == DialogResult.OK && applicationLoaded)
            {
                Program.comms.updateReferences();
                coreService = Program.comms.getCoreService();
                ontologyService = Program.comms.getOntologyService();
                governanceService = Program.comms.getGovernanceService();

                getRegistryInfo();
            }
            
            this.Cursor = Cursors.Default;
        }

        // Exit the application
        private void exitToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Application.Exit();
        }

        // Show the "Create a Single Document" dialog.
        private void singleDocumentToolStripMenuItem_Click(object sender, EventArgs e)
        {
            this.Cursor = Cursors.WaitCursor;
            CreateSingleDocumentForm form = new CreateSingleDocumentForm();
            form.ShowDialog();
            this.Cursor = Cursors.Default;
        }

        // Show the "Create a Collection" dialog.
        private void collectionToolStripMenuItem_Click(object sender, EventArgs e)
        {
            this.Cursor = Cursors.WaitCursor;
            CreateCollectionForm form = new CreateCollectionForm();
            form.ShowDialog();
            this.Cursor = Cursors.Default;
        }

        //Show the "Create a Generic Object" dialog
        private void genericObjectToolStripMenuItem_Click(object sender, EventArgs e)
        {
            this.Cursor = Cursors.WaitCursor;
            CreateGOForm form = new CreateGOForm();
            form.ShowDialog();
            this.Cursor = Cursors.Default;
        }
 
        // Load the ontology data and all saved queries from WSRR
        private void getRegistryInfo()
        {
            // Load the ontologies and saved queries from WSRR. If we fail
            // to load the ontologies then don't load the queries.
            if (loadOntologies())
                loadQueries();
        }

        // Populates the tree view with all the ontologies found on the server. 
        private bool loadOntologies(){

                bool result = true;
                this.Cursor = Cursors.WaitCursor;
                try
                {
                    classificationsTreeView.Nodes.Clear();
                    OntologySystem[] ontologySystems = ontologyService.getSystems("en");

                    if (ontologySystems != null)
                    {
                        if (ontologySystems.Length > 0)
                        {
                            foreach (OntologySystem ontSys in ontologySystems)
                            {
                                TreeNode sysNode = new TreeNode(ontSys.name);
                                // Save the uri as the TreeNode's Tag for use in building xPath
                                sysNode.Tag = ontSys.uri;
                                
                                classificationsTreeView.Nodes.Add(sysNode);
                                if (ontSys.notEmpty)
                                {
                                    // Need to add children so that node can be expanded.
                                    OntologyClass[] rootClasses = ontologyService.getRootClassesForSystem(ontSys.uri, "en");
                                    if (rootClasses != null)
                                    {
                                        if (rootClasses.Length > 0)
                                        {
                                            foreach (OntologyClass ontCls in rootClasses)
                                            {
                                                TreeNode clsNode = new TreeNode(ontCls.name);
                                                // Save the uri as the TreeNode's Tag for use in building xPath
                                                clsNode.Tag = ontCls.uri;
                                                sysNode.Nodes.Add(clsNode);                                                
                                                addSubclasses(ontCls.uri, clsNode);
                                                
                                            }
                                        }
                                    }
                                }
                            }
                        }
                    }

                    // Now copy the hidden TreeView we have constructed into the 2 visible ones.
                    // We do this to avoid hitting WSRR twice for each ontology class.
                    GUI_Utilities.cloneTreeViewNodes(queryClassificationsTreeView.Nodes,classificationsTreeView);
                    GUI_Utilities.cloneTreeViewNodes(retrievedClassificationsTreeView.Nodes,classificationsTreeView);                    
                }
                catch (Exception ex)
                {
                    Program.comms.HandleException("Cannot retrieve ontologies", ex);
                    result = false;
                }                    
                finally
                {
                    this.Cursor = Cursors.Default;
                }

                return result;
            }

        //Utility method used by loadOntologies to build up the tree view of an ontology.
        private void addSubclasses(String classURI, TreeNode node)
        {
            OntologyClass[] childClasses = ontologyService.getImmediateSubclassesForClass(classURI, "en");
            if (childClasses != null)
            {
                if (childClasses.Length > 0)
                {
                    foreach (OntologyClass childClass in childClasses)
                    {
                        TreeNode newNode = new TreeNode(childClass.name);

                        // Set the Tag value to be the actual uri that we will use to build up
                        // the xPath expression.
                        newNode.Tag = childClass.uri;
                        node.Nodes.Add(newNode);

                        //Recurse along the tree adding the children of the new node
                        addSubclasses(childClass.uri, newNode);

                    }
                }
            }
        }

        private void aboutToolStripMenuItem_Click(object sender, EventArgs e)
        {
            AboutForm about = new AboutForm();
            about.ShowDialog();
        }

/******************************************************************************
 *                      Query Tab                                             *
 *****************************************************************************/

        // Search WSRR for all GraphQuery and PropertyQuery objects. Build up two lists, one of queries
        // without any text saved in the queryExpression attribute (named queries), one with queries
        // that have text in queryExpression (user defined queries), and place them into the 
        // relevamt combo boxes on the query tab.
        private bool loadQueries()
        {
            bool result = true;

            this.Cursor = Cursors.WaitCursor;
            try
            {

                DataGraphType[] graphResults = Utilities.runGraphQuery(coreService, "/WSRR/GraphQuery", 0, false);
                DataGraphType[] propertyResults = Utilities.runGraphQuery(coreService, "/WSRR/PropertyQuery", 0, false);


                // Concat the 2 result sets into one result set of all queries
                DataGraphType[] results = new DataGraphType[graphResults.Length + propertyResults.Length];
                graphResults.CopyTo(results, 0);
                propertyResults.CopyTo(results, graphResults.Length);

                // Clear any previous results in the combo boxes
                executeQueryNameComboBox.Items.Clear();
                loadSaveQueryNameComboBox.Items.Clear();

                for (int i = 0; i < results.Length; i++)
                {
                    DataGraphType result1 = results[i];
                    if (result1 != null)
                    {
                        BaseObject baseObject = Utilities.getRootObject(result1);
                        if (baseObject != null)
                            if (((QueryObject)baseObject).queryExpression != null)
                            {
                                loadSaveQueryNameComboBox.Items.Add(baseObject.name);
                            }
                            else
                                executeQueryNameComboBox.Items.Add(baseObject.name);
                    }
                }
            }
            catch (Exception e)
            {
                Program.comms.HandleException("Cannot retrieve queries", e);
                result = false;
            }            
            finally {
                this.Cursor = Cursors.Default;                
            }

            return result;
        }

        // Execute the user defined query, composed of the xPath expression,
        // the query type (graph or property), and the query options.
        private void executeQueryButton_Click(object sender, EventArgs e)
        {
            this.Cursor = Cursors.WaitCursor;

            DataGraphType[] queryResults = null;

            // Reset the bsrURIColumnIndex to -1. This disables the retrieve & update button and also the 
            // delete button. If the query returns the bsrURI as one of the columns then this will get re-enabled.
            bsrURIColumnIndex = -1;

            if (graphQueryRadioButton.Checked)
            {
                // It's a graph query
                try
                {
                    queryResults = Utilities.runGraphQuery(coreService, xpathTextBox.Text, Convert.ToInt32(queryDepthComboBox.SelectedItem.ToString()), !trueRadioButton.Checked);
                }
                catch (Exception we)
                {
                    Program.comms.HandleException("Could not execute the query", we);
                    this.Cursor = Cursors.Default;
                    return;
                }

                // Construct the DataTable programmatically
                DataTable queryResultsDataTable = new DataTable();

                if (queryResults != null)
                {
                    if (queryResults.Length > 0)
                    {
                        // Populate the DataTable programmatically
                        bsrURIColumnIndex = GUI_Utilities.addColumns(queryResultsDataTable);

                        // Now build the rows
                        GUI_Utilities.fillDataTable(queryResultsDataTable, queryResults);                        
                    }
                    else
                    {
                        MessageBox.Show("Query returned zero results");
                        this.Cursor = Cursors.Default;
                    }
                }
                else
                {
                    MessageBox.Show("results == null");
                    this.Cursor = Cursors.Default;
                }

                // Always assign the results to the DataGridView, so if we retrieve 0 rows
                // then we wipe the last results
                dataGridView1.DataSource = queryResultsDataTable;
            }
            else if (propertyQueryRadioButton.Checked)
            {
                // This is a Property Query. Display the properties the user has asked for.
                string[] propertiesArray = propertiesTextBox.Lines;

                if (propertiesArray != null)
                {
                    //Remove any blank lines from the array
                    List<String> p = new List<String>();
                    for (int c = 0; c < propertiesArray.Length; c++)
                        if (propertiesArray[c].Length > 0)
                            p.Add(propertiesArray[c]);

                    propertiesArray = p.ToArray();
                }

                if (propertiesArray != null && propertiesArray.Length > 0)
                {

                    try
                    {
                        queryResults = Utilities.runPropertyQuery(coreService, xpathTextBox.Text, propertiesArray);
                    }
                    catch (Exception we)
                    {
                        Program.comms.HandleException("Could not execute the query", we);
                        this.Cursor = Cursors.Default;
                        return;
                    }

                    // Construct the DataTable programmatically
                    DataTable queryResultsDataTable = new DataTable();

                    if (queryResults != null)
                    {
                        if (queryResults.Length > 0)
                        {
                            // Populate the DataTable programmatically
                            // One column per parameter, type is always string
                            // Ignoring xsdTargetNamespaces for now
                            for (int i = 0; i < propertiesArray.Length; i++)
                            {
                                DataColumn dataColumn = new DataColumn(propertiesArray[i], typeof(string));
                                queryResultsDataTable.Columns.Add(dataColumn);

                                // Make a note of the bsrURI if it has been retrieved. We can then delete or retrieve/update
                                if (propertiesArray[i] == "bsrURI")
                                    bsrURIColumnIndex = i;
                            }

                            // Now build the rows
                            GUI_Utilities.fillDataTable(queryResultsDataTable, queryResults);
                        }
                        else
                        {
                            MessageBox.Show("Query returned zero results");
                            this.Cursor = Cursors.Default;
                        }
                    }
                    else
                    {
                        MessageBox.Show("results == null");
                        this.Cursor = Cursors.Default;
                    }
                    // Always assign the results to the DataGridView, so if we retrieve 0 rows
                    // then we wipe the last results
                    dataGridView1.DataSource = queryResultsDataTable;

                }
                else
                {
                    MessageBox.Show("No properties specified");
                }
            }

            this.Cursor = Cursors.Default;
        }

        // Save the query. If a query will the same name already exists then prompt the user to over-write.
        // If they still want to save then we delete the old query first and then save the query as normal.
        private void saveQueryButton_Click(object sender, EventArgs e)
        {
            this.Cursor = Cursors.WaitCursor;

            DataGraphType[] existsQuery = Utilities.loadQuery(coreService, loadSaveQueryNameComboBox.Text);

            // If null was returned then there was an error communicating with WSRR so just return
            if (existsQuery == null)
                return;

            // If the query already exists then prompt the user to overwrite it
            if (existsQuery.Length > 0)
            {
                DialogResult res = MessageBox.Show("A query with name '" + loadSaveQueryNameComboBox.Text + "' already exists. Do you want to overwrite the existing query?", "Overwrite", MessageBoxButtons.YesNo);

                if (res == DialogResult.Yes)
                {
                    // Delete the query
                    coreService.delete(Utilities.getRootObject(existsQuery[0]).bsrURI);
                }
                else
                {
                    this.Cursor = Cursors.Default;
                    return;
                }
            }

            BaseObject query = null;

            if (graphQueryRadioButton.Checked)
            {
                // Create a graph query
                query = Utilities.createGraphQuery(xpathTextBox.Text, "_1", Convert.ToInt32(queryDepthComboBox.SelectedItem.ToString()), ! trueRadioButton.Checked);                
            }
            else
            {
                // Create a property query
                query = Utilities.createPropertyQuery(xpathTextBox.Text, "_1", propertiesTextBox.Lines);
            }

            query.name = loadSaveQueryNameComboBox.Text;

            create createRequest = new wsrrCoreAPI.create();
            createRequest.datagraph = new DataGraphType();
            createRequest.datagraph.WSRR = new WSRR();
            createRequest.datagraph.WSRR.artefacts = new BaseObject[1];
            createRequest.datagraph.WSRR.artefacts[0] = query;
            createRequest.datagraph.WSRR.root = "_1";

            try
            {
                createResponse createResponse = coreService.create(createRequest);
            }
            catch (Exception we)
            {
                Program.comms.HandleException("Could not save the query", we);
                return;
            }
            finally
            {
                this.Cursor = Cursors.Default;
            }

            MessageBox.Show("Query saved successfully");

            // Repopulate the list of named queries from WSRR
            loadQueries();
        }

        // Execute one of the named queries that ship with WSRR. Supply any parameters that have been provided
        // by the user
        private void buttonExecuteSpecifiedNamedQuery_Click(object sender, EventArgs e)
        {
            this.Cursor = Cursors.WaitCursor;

            DataGraphType[] results = null;

            // Check to see if any parameters have been supplied.
            string[] parameters = parametersTextBox.Lines;

            if (parameters != null)
            {
                //Remove any blank lines from the array
                List<String> p = new List<String>();
                for (int c = 0; c < parameters.Length; c++)
                    if (parameters[c].Length > 0)
                        p.Add(parameters[c]);

                parameters = p.ToArray();
            }

            try
            {
                if (parameters != null && parameters.Length > 0)
                {
                    results = coreService.executeNamedQueryWithParameters(executeQueryNameComboBox.Text, parameters);
                }
                else
                {
                    results = coreService.executeNamedQuery(executeQueryNameComboBox.Text);
                }
                
                
            }
            catch (Exception we)
            {
                Program.comms.HandleException("Could not execute the named query", we);
                this.Cursor = Cursors.Default;
                return;
            }

            // Construct the DataTable programmatically
            DataTable resultsDataTable = new DataTable();

            if (results != null)
            {
                if (results.Length > 0)
                {                    
                    bsrURIColumnIndex = GUI_Utilities.addColumns(resultsDataTable);

                    // Now build the rows
                    GUI_Utilities.fillDataTable(resultsDataTable, results);
                }
                else
                {
                    MessageBox.Show("Query returned zero results");
                    this.Cursor = Cursors.Default;
                }
            }
            else
            {
                MessageBox.Show("results == null");
                this.Cursor = Cursors.Default;
            }

            // Associate results with DataGridView.
            dataGridView1.DataSource = resultsDataTable;

            this.Cursor = Cursors.Default;
        }

        // Toggle the visibility of the group boxes that contain the relevant
        // parameters for either a Graph query or Property query. Set the Enabled state
        // of the "Execute" button to true if the user has provided all necessary information.
        private void graphPropertyQueryRadioButton_CheckedChanged(object sender, EventArgs e)
        {
            if (graphQueryRadioButton.Checked)
            {
                graphQueryGroupBox.Visible = true;
                propertyQueryOptionsGroupBox.Visible = false;
            }
            else if (propertyQueryRadioButton.Checked)
            {
                graphQueryGroupBox.Visible = false;
                propertyQueryOptionsGroupBox.Visible = true;
            }

            checkCanExecuteQuery(sender,null);
        }      

        // Load the user defined query from WSRR and populate the relevant
        // parameters on the form.
        private void loadNamedQueryButton_Click(object sender, EventArgs e)
        {

            this.Cursor = Cursors.WaitCursor;
            DataGraphType[] result = Utilities.loadQuery(coreService, loadSaveQueryNameComboBox.Text);
            this.Cursor = Cursors.Default;

            //Have we found a query?
            if (result == null || result.Length == 0)
            {
                MessageBox.Show("No query found with name '" + loadSaveQueryNameComboBox.Text + "'");
            }
            else
            {
                //There should just be one object returned.
                if (result.Length > 1)
                {
                    MessageBox.Show(result.Length + " queries found with name '" + loadSaveQueryNameComboBox.Text + "'. Only the first will be loaded");
                }

                // Set this to true so the classifications tree view and the xPath text box don't interfere
                // when trying to set all the parameters up for the query.
                loadingQuery = true;

                // Get the query from the DataGraphType
                QueryObject query = (QueryObject)Utilities.getRootObject(result[0]);

                // Set the query expression
                xpathTextBox.Text = query.queryExpression;

                if (query is GraphQuery)
                {                    
                    graphQueryRadioButton.Checked = true;

                    //Set the depth
                    if (((GraphQuery)query).depth == -1)
                        queryDepthComboBox.SelectedIndex = 0;
                    else if (((GraphQuery)query).depth == 0)
                        queryDepthComboBox.SelectedIndex = 1;
                    else
                        queryDepthComboBox.SelectedIndex = 2;

                    // the noContent setting is not persisted so we cannot set the
                    // "Retrieve Document Content" check boxes                        
                }
                else
                {
                    propertyQueryRadioButton.Checked = true;

                    // Copy the property query parameters into the textbox. USe an intermmediate array
                    // so we don't copy any user defined properties that are not part of the property query
                    // and also so we get the properties in the correct order.
                    propertiesTextBox.Text = "";
                    String[] props = new String[query.userDefinedProperties.Length];
                    for (int c = 0; c < query.userDefinedProperties.Length; c++)
                        if (query.userDefinedProperties[c].name.StartsWith("PropertyQueryProperty"))
                            props[Convert.ToInt32(query.userDefinedProperties[c].name.Replace("PropertyQueryProperty", ""))] = query.userDefinedProperties[c].value;

                    for (int c = 0; c < props.Length; c++)
                        if (props[c] != null && props[c].Length > 0)
                            propertiesTextBox.Text += props[c] + "\r\n";
                }

                // Set up the classifications tree view to reflect the xPath for the 
                // classification conditions
                GUI_Utilities.resetCheckBoxes(queryClassificationsTreeView);
                fillInClassificationsTreeView(xpathTextBox.Text);

                // Now the tree view and the xPath box can change each other if either
                // one of them is edited.
                loadingQuery = false;
            }
        }

        // This is called when the user either clicks the "Delete" button, or selects "Delete" from
        // the context menu for the query results DataGridView.
        private void toolStripMenuItem_Delete_Click(object sender, EventArgs e)
        {
            // Make a copy of the selected rows so we can delete them one by one
            int count = dataGridView1.SelectedRows.Count;
            DataGridViewRow[] rows = new DataGridViewRow[count];
            dataGridView1.SelectedRows.CopyTo(rows, 0);

            if (MessageBox.Show("Are you sure you want to delete these " + count + " items?", "Delete items", MessageBoxButtons.YesNo) == DialogResult.Yes)
            {
                this.Cursor = Cursors.WaitCursor;
                try
                {
                    // Delete both the document from the registry and the row from the DataGridView
                    int c = 0;
                    for (c = 0; c < rows.Length; c++)
                    {
                        coreService.delete(rows[c].Cells[bsrURIColumnIndex].Value.ToString());
                        dataGridView1.Rows.Remove(rows[c]);
                    }

                }
                catch (Exception we)
                {
                    Program.comms.HandleException("Could not delete the document", we);
                    return;
                }
                finally
                {
                    this.Cursor = Cursors.Default;
                }

                MessageBox.Show("Delete was successful");
            }
        }

        // This is called when the user either clicks the "Retrieve and Update" button, or selects "Retrieve and Update" from
        // the context menu for the query results DataGridView. We only Retrieve the currently selected item and do
        // not support multiple items for this operation.
        private void retrieveAndUpdateToolStripMenuItem_Click(object sender, EventArgs e)
        {

            this.Cursor = Cursors.WaitCursor;
            
            //Set bsrURI field on the retrieve tab to be the required bsrURI and then
            //activate the retrieve tab
            retrieveBsrURITextBox.Text = dataGridView1.SelectedRows[0].Cells[bsrURIColumnIndex].Value.ToString();
            tabControl1.SelectedTab = retrieve;
            retrieveDepthComboBox.SelectedIndex = 0;
            trueRadioButton.Select();

            // load the object up.
            retrieveButton_Click(null, null);
            
            this.Cursor = Cursors.Default;
        }

        // If some rows are selected in the query results DataGridView, and the
        // bsrURI is present in one of the columns then enabled the "Retrieve and Update" 
        // and "Delete" buttons and context menu items. Otherwise disable them.
        private void dataGridView1_SelectionChanged(object sender, EventArgs e)
        {
            if (dataGridView1.SelectedRows.Count > 0 && bsrURIColumnIndex > -1)
            {
                retrieveAndUpdateButton.Enabled = true;
                DeleteButton.Enabled = true;
                queryResultsContextMenuStrip.Enabled = true;
            }
            else
            {
                retrieveAndUpdateButton.Enabled = false;
                DeleteButton.Enabled = false;
                queryResultsContextMenuStrip.Enabled = false;
            }
        }

        // This function decides whether we can enable the "Execute Query" button on the query tab.
        // The following conditions must be met for the button to be enabled:
        //
        // - XPath Expression must not be empty
        // - Either the "Graph query" or "Property query" radio button is selected
        // - If "Property query" is selected then there is at least one character in the properties text box
        // - If "Graph query" is selected then there is always a default for depth and noContent so this ok.
        private void checkCanExecuteQuery(object sender, EventArgs e)
        {

            string[] propertiesArray = propertiesTextBox.Lines;

            if (xpathTextBox.Text.Length > 0 &&
                (graphQueryRadioButton.Checked ||
                    (propertyQueryRadioButton.Checked && propertiesArray != null && propertiesArray.Length > 0)
                )
                )
            {
                executeQueryButton.Enabled = true;
            }
            else
            {
                executeQueryButton.Enabled = false;
            }

            //If we are not already in the middle of an update to the classifications tree or xPath expression,
            // i.e. loading Query == false, then update the classifications tree to match the xPath expression.
            if (loadingQuery == false)
            {
                loadingQuery = true;
                fillInClassificationsTreeView(xpathTextBox.Text);
                loadingQuery = false;
            }
        }

        // If named query has been selected then enable the "Execute Named Query" button
        private void executeQueryNameComboBox_SelectedIndexChanged(object sender, EventArgs e)
        {
            buttonExecuteSpecifiedNamedQuery.Enabled = true;
        }

        //If the user has entered a name then enable the "Load query" and "Save query" buttons
        private void loadSaveQueryNameComboBox_TextChanged(object sender, EventArgs e)
        {
            if (loadSaveQueryNameComboBox.Text.Length > 0)
            {
                loadNamedQueryButton.Enabled = true;
                saveQueryButton.Enabled = true;
            }
            else
            {
                loadNamedQueryButton.Enabled = false;
                saveQueryButton.Enabled = false;
            }
           
        }
        // This called when the user has changed the "Checked" state of one of the TreeView nodes on the query tab.
        // It updates the "XPath expression" text box with the new classification conditions
        private void queryClassificationsTreeView_AfterCheck(object sender, TreeViewEventArgs e)
        {
            String[] classes = GUI_Utilities.getTreeViewSelectedUris(queryClassificationsTreeView);

            // Only do this if the state change was due to the user and not due to the XPath text box updating the tree view
            // We set loadingQuery to true when a non-user update is being performed to either the tree view or the xpath text box
            if (loadingQuery == false)
            {
                loadingQuery = true;
                xpathTextBox.Text = Utilities.filterByClassification(xpathTextBox.Text, classes, allOfRadioButton.Checked, MatchChildrenCheckBox.Checked);
                loadingQuery = false;
            }
        }

        // Called either of the 2 radiobuttons are clicked, or when "Match Children" is clicked.
        // This just calls queryClassificationsTreeView_AfterCheck() to update the XPath text box.
        private void queryClassificationsButtons_Click(object sender, EventArgs e)
        {
            queryClassificationsTreeView_AfterCheck(null, null);
        }

        // This is called after the user has chanegd the XPath expression. It process the string and
        // updates the tree view so that they are both kept in sync.
        public void fillInClassificationsTreeView(String xPath)
        {
            // All possible classification phrases
            String[] phrases = new string[] { "classifiedByAllOf(", "classifiedByAnyOf(", "exactlyClassifiedByAllOf(", "exactlyClassifiedByAnyOf(" };

            // Make sure the function to translate tree view changes to xpath expression doesn't interfere with us
            loadingQuery = true;

            // Find the classification phrase that is being used
            int c = -1;
            int index = -1;
            while (index == -1 && ++c < phrases.Length)
            {
                index = xPath.IndexOf(phrases[c]);
            }

            // If we have found a classification phrase then setup the buttons in the classifications group box
            if (index != -1)
            {
                switch (c)
                {
                    case 0:
                        //classifiedByAllOf
                        allOfRadioButton.Checked = true;
                        anyOfRadioButton.Checked = false;
                        MatchChildrenCheckBox.Checked = true;
                        break;
                    case 1:
                        //classifiedByAnyOf
                        allOfRadioButton.Checked = false;
                        anyOfRadioButton.Checked = true;
                        MatchChildrenCheckBox.Checked = true;
                        break;
                    case 2:
                        //exactlyClassifiedByAllOf
                        allOfRadioButton.Checked = true;
                        anyOfRadioButton.Checked = false;
                        MatchChildrenCheckBox.Checked = false;
                        break;
                    case 3:
                        //exactlyClassifiedByAnyOf
                        allOfRadioButton.Checked = false;
                        anyOfRadioButton.Checked = true;
                        MatchChildrenCheckBox.Checked = false;
                        break;
                }

                // Create an array of all the classifications
                String classes = xPath.Substring(index + phrases[c].Length).Replace("\'", "").Replace("]", "").Replace(")", "");
                String[] classifications = classes.Split(',');

                // Update the tree view checkboxes to match the XPath expression
                GUI_Utilities.resetCheckBoxes(queryClassificationsTreeView);
                GUI_Utilities.setCheckBoxes(queryClassificationsTreeView, classifications);
            }
            else
                // If there was no classification conditions then remove all the Checked state from 
                // all the tree view nodes
                GUI_Utilities.resetCheckBoxes(queryClassificationsTreeView);

            loadingQuery = false;
        }

/******************************************************************************
 * Retrieve tab                                                               *
 *****************************************************************************/

        // Load the document that matches the specified bsrURI from WSRR
        private void retrieveButton_Click(object sender, EventArgs e)
        {
            DataGraphType retrievedDG = null;

            this.Cursor = Cursors.WaitCursor;

            try
            {
                // Always retrieve the document with the specified depth and noContent setting
                Options options = new Options();
                options.noContent = ! trueRadioButton.Checked;
                options.depth = Convert.ToInt32(retrieveDepthComboBox.Text);
                retrievedDG = coreService.retrieveWithOptions(retrieveBsrURITextBox.Text,options);
            }
            catch (Exception we)
            {
                Program.comms.HandleException("Could not retrieve the document", we);
                this.Cursor = Cursors.Default;
                return;
            }

            // Disable "Update" button, we will enable it firther down if 
            // we successfully find the object
            updateButton.Enabled = false;

            // Find the object we are interested in, inside the DataGraphType
            retrievedObject = Utilities.getRootObject(retrievedDG);

            if (retrievedObject != null)
            {

                // Display object type and bsrURI
                String obType = retrievedObject.GetType().ToString();
                fileTypeLabel.Text = obType.Substring(obType.LastIndexOf(".") + 1);
                bsrURILabel.Text = retrievedObject.bsrURI;

                // Display modeled properties
                docNameTextBox.Text = retrievedObject.name;
                docNamespaceTextBox.Text = retrievedObject.@namespace;
                docVersionTextBox.Text = retrievedObject.version;
                docDescriptionTextBox.Text = retrievedObject.description;

                // Update the governance info labels and combo boxes
                displayGovernanceInfo();

                // Display any existing user-defined properties
                userPropertiesDataGridView.Rows.Clear();
                UserDefinedProperty[] udps = retrievedObject.userDefinedProperties;
                if (udps != null)
                {
                    if (udps.Length > 0)
                    {
                        for (int i = 0; i < udps.Length; i++)
                        {
                            String[] values = new String[2];
                            UserDefinedProperty udp = new UserDefinedProperty();
                            udp = udps[i];
                            values[0] = udp.name;
                            values[1] = udp.value;
                            userPropertiesDataGridView.Rows.Add(values);
                        }
                    }
                }

                // Reset the classifications tree view
                retrievedClassificationsTreeView.CollapseAll();
                GUI_Utilities.resetCheckBoxes(retrievedClassificationsTreeView);
                
                // Loop through the classifications set for the retrieved  object and then
                // set the Checked attribute for the corresponding tree view node.
                if (retrievedObject.classificationURIs != null)
                    foreach (String classification in retrievedObject.classificationURIs)
                    {
                        TreeNode tn = GUI_Utilities.findTreeNodeWithTag(retrievedClassificationsTreeView,classification);

                        if (tn != null)
                        {
                            tn.Checked = true;

                            // We only want to expand the parts of the tree that are relevant to this object.
                            GUI_Utilities.expandParents(tn);
                        }
                    }

                //Enable all fields for editing
                docNameTextBox.Enabled = true;
                docNamespaceTextBox.Enabled = true;
                docVersionTextBox.Enabled = true;
                docDescriptionTextBox.Enabled = true;
                docDescriptionTextBox.Enabled = true;
                userPropertiesDataGridView.Enabled = true;

                // Set any fields to readOnly that should be        				
		        //namespace and version on XSD, WSDL and Policy are read only
	            if (retrievedObject is WSDLDocument || retrievedObject is XSDDocument || retrievedObject is PolicyDocument)
                {
                    docNamespaceTextBox.Enabled = false;
                    docVersionTextBox.Enabled = false;
		        }

                // If this is not a logical object then enable Update
                if (retrieveDepthComboBox.Text == "-1" && retrievedObject is LogicalObject == false)
                {
                    updateButton.Enabled = true;
                    updateButton.Text = "Update";
                    updateButton.Location = new System.Drawing.Point((this.Width - updateButton.Width) / 2,updateButton.Location.Y);
                }
                else
                {
                    // Disable editing and the update button for all logical objects
                    docNameTextBox.Enabled = false;
                    docNamespaceTextBox.Enabled = false;
                    docVersionTextBox.Enabled = false;
                    docDescriptionTextBox.Enabled = false;
                    docDescriptionTextBox.Enabled = false;
                    userPropertiesDataGridView.Enabled = false;
                    updateButton.Enabled = false;

                    if (retrievedObject is LogicalObject)
                        updateButton.Text = "Update is not supported for Logical Objects";
                    else                        
                        updateButton.Text = "Update is not supported unless depth is set to -1";

                    updateButton.Location = new System.Drawing.Point((this.Width - updateButton.Width) / 2, updateButton.Location.Y);
                }
            }
            else
            {
                MessageBox.Show("retrievedObject is null");
            }

            this.Cursor = Cursors.Default;
        }

        private void displayGovernanceInfo() {
                // Get governance detail
                gR = governanceService.getGovernanceRecord(retrievedObject.bsrURI);

                // Disable the forms objects. We will enable them further down if we need to
                governanceStateComboBox.Items.Clear();
                GovernButton.Enabled = false;
                RemoveGovernanceButton.Enabled = false;
                governanceStateComboBox.Enabled = false;

                if (gR != null)
                {
                    // Look up the translation of the governance state from the provided state
                    OntologyClass ontClass = ontologyService.getClass(gR.state,"en");
                    docGovernanceStateLabel.Text = ontClass.name;

                    // As this object is already governed then relabel the govern button to: Transition

                    GovernButton.Text = "&Transition";
                    //Populate the state transition governance drop down
                    String[] trans = null;
                    try
                    {
                        trans = governanceService.getTransitions(retrievedObject.bsrURI);
                        RemoveGovernanceButton.Enabled = true;

                        if (gR.governedObject.Length > 1)
                            // Flag that this is the parent member of a governed collection. Just to be extra informative
                            docGovernanceStateLabel.Text += " - parent member of a governed collection";
                    }
                    catch (Exception)
                    {
                        // If we get an exception then this must have been a child member of a
                        // governed collection. Flag this and the "Remove Governance" and "Transition" buttons will not be enabled
                        docGovernanceStateLabel.Text += " - child member of a governed collection";
                    }

                    // If we have some states we can transition to then add them to the combo box and enable the "Remove Governance"
                    // button. The GovernButton will be enabled when an item is selected from the combo box list.
                    if (trans != null)
                    {
                        foreach (String tran in trans)
                            governanceStateComboBox.Items.Add(new LabelId(ontologyService.getClass(tran, "en").name, tran));

                        governanceStateComboBox.Enabled = true;
                    }
                                        
                }
                else
                {
                    docGovernanceStateLabel.Text = "Ungoverned";

                    //Populate the initial state transition governance drop down
                    try
                    {
                        String[] trans = governanceService.getInitialTransitions(retrievedObject.bsrURI);

                        if (trans != null)
                            foreach (String tran in trans)
                                governanceStateComboBox.Items.Add(new LabelId(ontologyService.getClass(tran, "en").name, tran));

                        governanceStateComboBox.Items.Add("Default transition");

                        // As this object is not governed then make sure the govern button is labeled Govern
                        GovernButton.Text = "&Govern";
                        governanceStateComboBox.Enabled = true;
                    }
                    catch (Exception)
                    {
                        // Governance is not valid for this object type, so do not enable the buttons etc.
                        docGovernanceStateLabel.Text = "Not governable";
                    }
                }

                // If we managed to add any available states to the combo box then select the first one
                if (governanceStateComboBox.Enabled && governanceStateComboBox.Items != null && governanceStateComboBox.Items.Count > 0)
                    governanceStateComboBox.SelectedIndex = 0;
        }

        // If the "Update" button has been clicked then copy across any settings from the form
        // that the user has changed and issue the update call to WSRR.
        private void updateButton_Click(object sender, EventArgs e)
        {

            this.Cursor = Cursors.WaitCursor;

            // Look for changes to the modeled properties
            if (docNameTextBox.Text != retrievedObject.name)
            {
                retrievedObject.name = docNameTextBox.Text;
            }
            if (docNamespaceTextBox.Text != retrievedObject.@namespace)
            {
                retrievedObject.@namespace = docNamespaceTextBox.Text;
            }
            if (docVersionTextBox.Text != retrievedObject.version)
            {
                retrievedObject.version = docVersionTextBox.Text;
            }
            if (docDescriptionTextBox.Text != retrievedObject.description)
            {
                retrievedObject.description = docDescriptionTextBox.Text;
            }

            // Just copy across the current udps from the form            
            retrievedObject.userDefinedProperties = GUI_Utilities.generateUDP(userPropertiesDataGridView);

            // Set the classifications
            retrievedObject.classificationURIs = GUI_Utilities.getTreeViewSelectedUris(retrievedClassificationsTreeView);

            // Now issue the update request
            update updateRequest = new update();
            updateRequest.datagraph = new DataGraphType();
            updateRequest.datagraph.WSRR = new WSRR();
            updateRequest.datagraph.WSRR.artefacts = new BaseObject[1];
            updateRequest.datagraph.WSRR.artefacts[0] = retrievedObject;
            updateRequest.datagraph.WSRR.root = retrievedObject.bsrURI;

            try
            {
                updateResponse updateResponse = coreService.update(updateRequest);
            }
            catch (Exception we)
            {
                Program.comms.HandleException("Could not update the document", we);
                this.Cursor = Cursors.Default;
                return;
            }

            this.Cursor = Cursors.Default;

            MessageBox.Show("Update was successful");           
        }

        // This function makes sure the DataGridView always fills all the avilable space
        private void userPropertiesDataGridView_Resize(object sender, EventArgs e)
        {
            userPropertiesDataGridView.Columns[1].Width = userPropertiesDataGridView.Width - userPropertiesDataGridView.Columns[0].Width;
        }

        // This function makes sure the modelled properties boxes always make use of the available space
        private void groupBox6_Resize(object sender, EventArgs e)
        {
            docNameTextBox.Width = (groupBox6.Width - docNameTextBox.Left - 10) * 2 / 3;
            docVersionTextBox.Width = (groupBox6.Width - docVersionTextBox.Left - 10) * 2 / 3;
            docNamespaceTextBox.Width = (groupBox6.Width - docNamespaceTextBox.Left - 10) * 2 / 3;
        }

        // Enable or disable the "Retrieve and Update" button based on whether a bsrURI has been entered
        private void retrieveBsrURITextBox_TextChanged(object sender, EventArgs e)
        {
            if (retrieveBsrURITextBox.Text != null && retrieveBsrURITextBox.Text.Length > 0)
                retrieveButton.Enabled = true;
            else
                retrieveButton.Enabled = false;
        }

        // Called when the user clicks the "Govern" or "Transition" button.
        private void GovernButton_Click(object sender, EventArgs e)
        {            
            this.Cursor = Cursors.WaitCursor;
            
            if (gR == null)
            {
                // If we do not have a governance record currently loaded then this
                // object must be ungoverned, so call makeGovernable
                governanceService.makeGovernable(retrievedObject.bsrURI, "");
            }
            else
            {
                // If we have a governance record currently loaded then this
                // object must already be governed and so we need to transition it
                governanceService.transition(retrievedObject.bsrURI, ((LabelId)governanceStateComboBox.SelectedItem).id);
            }

            // Reload the governance info
            displayGovernanceInfo();

            this.Cursor = Cursors.Default;            
        }

        // If the user has selected a state from the governance state combo box then enable the "Govern" button
        private void governanceStateComboBox_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (governanceStateComboBox.SelectedItem != null)
                GovernButton.Enabled = true;
        }

        // When teh "Remove Governance" button is clicked then call the removeGovernance method on the governanceService
        private void RemoveGovernanceButton_Click(object sender, EventArgs e)
        {
            this.Cursor = Cursors.WaitCursor;
            governanceService.removeGovernance(retrievedObject.bsrURI);

            // Reload the governance info
            displayGovernanceInfo();
            this.Cursor = Cursors.Default;
        }
    }
}