/* begin_generated_IBM_copyright_prolog                            */
/* 
 * Licensed Materials - Property of IBM 
 * 
 * 5724-N72 5655-R41 
 * 
 * Copyright IBM Corp. 2006, 2007 All Rights Reserved 
 * 
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with 
 * IBM Corp. 
 */
/* end_generated_IBM_copyright_prolog                            */
using System;
using System.Collections.Generic;
using System.Data;
using System.Text;
using System.Text.RegularExpressions;
using WSRRSample.wsrrCoreAPI;
using System.Windows.Forms;
using System.Collections;

/* This module contains utility functions that simply interfacing with a WSRR instance*/
namespace WSRRSample
{
    class Utilities
    {
        // executes a graph query with the specified xPath expression, depth, and noContent setting
        public static DataGraphType[] runGraphQuery(WSRRCoreSDOService coreService, String xpath, int depth, bool content)
        {

            DataGraphType[] queryResults = null;

            GraphQuery graphQuery = createGraphQuery(xpath, "_1", depth, content);

            BaseObject[] artefacts = new BaseObject[1];
            artefacts[0] = graphQuery;
            DataGraphType queryDG = Utilities.createDataGraph(artefacts, "_1");
            
            queryResults = coreService.executeQuery(queryDG);

            return queryResults;
        }

        //creates a graph query with the specified xpath and depth
        public static GraphQuery createGraphQuery(String xpath, String rootID, int depth, bool content)
        {
            GraphQuery graphQuery = new GraphQuery();
            graphQuery.bsrURI = rootID;
            graphQuery.queryExpression = xpath;
            graphQuery.depth = depth;
            graphQuery.noContent = content;

            return graphQuery;
        }

        // executes a property query with the specified xPath expression and property names
        public static DataGraphType[] runPropertyQuery(WSRRCoreSDOService coreService, String xpath, String[] properties)
        {
            String rootID = "_1";
            PropertyQuery propertyQuery = createPropertyQuery(xpath, rootID, properties);
            BaseObject[] artefacts = new BaseObject[1];
            artefacts[0] = propertyQuery;
            DataGraphType queryDG = Utilities.createDataGraph(artefacts, rootID);

            DataGraphType[] queryResults = coreService.executeQuery(queryDG);
            return queryResults;
        }

        //creates a property query from the specified xpath and property names
        public static PropertyQuery createPropertyQuery(String xPath, String rootID, String[] properties)
        {
            wsrrCoreAPI.PropertyQuery propertyQuery = new wsrrCoreAPI.PropertyQuery();
            propertyQuery.bsrURI = rootID;
            propertyQuery.queryExpression = xPath;
            // Add a user-defined property for each property requested
            wsrrCoreAPI.UserDefinedProperty[] udps = new wsrrCoreAPI.UserDefinedProperty[properties.Length];
            for (int i = 0; i < properties.Length; i++)
            {
                wsrrCoreAPI.UserDefinedProperty udp = new wsrrCoreAPI.UserDefinedProperty();
                udp.name = "PropertyQueryProperty" + i.ToString();
                udp.value = properties[i];
                udps[i] = udp;
            }
            propertyQuery.userDefinedProperties = udps;

            return propertyQuery;
            
        }

        // prints out the contents of a query result set to the console
        public static void printQueryResult(DataGraphType[] results)
        {
            Console.WriteLine("Number of results: " + results.Length.ToString());
            foreach (DataGraphType dg in results)
            {
                printRootObject(dg);
            }
        }

        // prints out the root object of a DataGraphType
        public static void printRootObject(DataGraphType dg)
        {
            BaseObject baseObject = getRootObject(dg);
            if (baseObject != null)
            {
                Console.Write("[");	// '{' has special meaning in C#
                Console.Write(baseObject.name);
                Console.Write(", ");
                Console.Write(baseObject.@namespace); // namespace is a reserved word in C#
                Console.Write(", ");
                Console.Write(baseObject.version);
                Console.WriteLine("]");	// '}' has special meaning in C#
            }
        }

        // Gets the root object from a DataGraphType
        public static BaseObject getRootObject(DataGraphType dg)
        {
            BaseObject result = null;
            // Check for WSRR instance as the root object first
            WSRR wsrr = dg.WSRR;
            if (wsrr != null)
            {
                String rootID = wsrr.root;
                // find the object or sub-object that matches the root bsrURI
                result = findMatchingSubObject(wsrr.artefacts, rootID);
            }
            else
            {
                // Looks like we have been given a DataGraph containing a property query result
            }
            return result;
        }

        // Finds the object in baseObjectA that matches bsrURI
        private static BaseObject findMatchingSubObject(BaseObject[] baseObjectA, string bsrURI)
        {
            foreach (BaseObject bo in baseObjectA)
            {
                BaseObject found = findMatchingSubObject(bo, bsrURI);
                if (found != null)
                    return found;
            }

            return null;
        }

        // Finds the contained object in baseObject that matches bsrURI. First checks that
        // the passed object does not match and then follows all containement relationships.
        private static BaseObject findMatchingSubObject(BaseObject baseObject, string bsrURI)
        {
            // Either this object matches the bsrURI or an object contained in a relationship
            // of this object matches the bsrURI. We check which type of object we have and then
            // check the relevant relationships.
            BaseObject bo = null;

            if (baseObject.bsrURI == bsrURI)
                bo = baseObject;
            else if (baseObject is WSDLMessage)
            {
                WSDLMessage m = (WSDLMessage)baseObject;
                bo = findMatchingSubObject(m.messageParts, bsrURI);
            }
            else if (baseObject is WSDLPortType)
            {
                WSDLPortType m = (WSDLPortType)baseObject;
                bo = findMatchingSubObject(m.operations, bsrURI);
            }
            else if (baseObject is WSDLService)
            {
                WSDLService m = (WSDLService)baseObject;
                bo = findMatchingSubObject(m.ports, bsrURI);
            }
            else if (baseObject is WSDLPort)
            {
                WSDLPort m = (WSDLPort)baseObject;
                bo = findMatchingSubObject(m.SOAPAddress, bsrURI);
            }
            else if (baseObject is WSDLBinding)
            {
                WSDLBinding m = (WSDLBinding)baseObject;
                bo = findMatchingSubObject(m.SOAPBinding, bsrURI);
            }
            else if (baseObject is Module)
            {
                Module m = (Module)baseObject;
                bo = findMatchingSubObject(m.exports, bsrURI);
                if (bo == null)
                    bo = findMatchingSubObject(m.imports, bsrURI);
            }
            else if (baseObject is Export)
            {
                Export m = (Export)baseObject;
                bo = findMatchingSubObject(m.exportBinding, bsrURI);
                if (bo == null)
                    bo = findMatchingSubObject(m.interfaces, bsrURI);
            }
            else if (baseObject is Import)
            {
                Import m = (Import)baseObject;
                bo = findMatchingSubObject(m.importBinding, bsrURI);
                if (bo == null)
                    bo = findMatchingSubObject(m.interfaces, bsrURI);
            }
            return bo;
        }

        // creates a DataGraphType. Used by runGraphQuery and runPropertyQuery.
        public static DataGraphType createDataGraph(BaseObject[] objects, string rootID)
        {
            WSRR wsrr = new WSRR();
            wsrr.artefacts = objects;
            wsrr.root = rootID;
            DataGraphType dg = new DataGraphType();
            dg.WSRR = wsrr;
            return dg;
        }


        // Builds a hashtable [bsrURI -> baseObject] for all objects in the DataGraphType
        static Hashtable buildHashtable(DataGraphType dg)
        {
            Hashtable hm = new Hashtable();
            BaseObject[] topLevelObjects = dg.WSRR.artefacts;
            mapArrayOfBaseObjects(hm, topLevelObjects);
            return hm;
        }

        // Call mapBaseObject(Hashtable, Basebject) for each object in baseObjects
        private static void mapArrayOfBaseObjects(Hashtable hm,
                                                  BaseObject[] baseObjects)
        {
            foreach (BaseObject baseObject in baseObjects)
            {
                mapBaseObject(hm, baseObject);
            }
        }

        // Add the baseObject and contained objects to the Hasttable
        private static void mapBaseObject(Hashtable hm, BaseObject baseObject)
        {
            String bsrURI = baseObject.bsrURI;
            if (hm.Contains(bsrURI) == false)
            {
                hm.Add(bsrURI, baseObject);
            }
            /*
             * Now follow containment relationships, recursing to add either a
             * single object or an array of objects as appropriate.
             */
            if (baseObject is ComplexTypeDefinition)
            {
                ComplexTypeDefinition ctd = (ComplexTypeDefinition)baseObject;
                mapArrayOfBaseObjects(hm, ctd.localAttributes);
            }
            else if (baseObject is Export)
            {
                Export exp = (Export)baseObject;
                mapBaseObject(hm, exp.exportBinding);
                mapArrayOfBaseObjects(hm, exp.interfaces);
            }
            else if (baseObject is Import)
            {
                Import imp = (Import)baseObject;
                mapBaseObject(hm, imp.importBinding);
                mapArrayOfBaseObjects(hm, imp.interfaces);
            }
            else if (baseObject is Module)
            {
                Module module = (Module)baseObject;
                mapArrayOfBaseObjects(hm, module.imports);
                mapArrayOfBaseObjects(hm, module.exports);
            }
            else if (baseObject is WSDLBinding)
            {
                mapBaseObject(hm, ((WSDLBinding)baseObject).SOAPBinding);
            }
            else if (baseObject is WSDLMessage)
            {
                mapArrayOfBaseObjects(hm,
                                 ((WSDLMessage)baseObject).messageParts);
            }
            else if (baseObject is WSDLPort)
            {
                mapBaseObject(hm, ((WSDLPort)baseObject).SOAPAddress);
            }
            else if (baseObject is WSDLPortType)
            {
                mapArrayOfBaseObjects(hm,
                                  ((WSDLPortType)baseObject).operations);
            }
            else if (baseObject is WSDLService)
            {
                mapArrayOfBaseObjects(hm, ((WSDLService)baseObject).ports);
            }
        }

        // Given a full file name this function will look at the extension
        // and then return a String that represents that file type
        // which we display in the GUI
        public static String getFileType(String file)
        {
            String ext = file.Substring(file.LastIndexOf(".") + 1);
            String fType;

            switch (ext.ToLowerInvariant())
            {
                case "wsdl": fType = "WSDL"; break;
                case "xml": fType = "XML"; break;
                case "xsd": fType = "XSD"; break;
                case "policy": fType = "WS-POLICY"; break;
                case "zip": fType = "SCA"; break;
                default: fType = "Other"; break;
            }

            return fType;
        }

        // Given an SDO Document object this function
        // returns a human readable type name
        public static String getDocumentType(Document doc)
        {
            if (doc is WSDLDocument)
                return "WSDL Document";
            else if (doc is XSDDocument)
                return "XSD Document";
            else if (doc is XMLDocument)
                return "XML Document";
            else if (doc is PolicyDocument)
                return "Policy Document";
            else if (doc is ModuleDocument)
                return "SCA Module";
            else if (doc is GenericDocument)
                return "Other Document";
            else
                return "Generic Object";
        }

        // Given a String returned by getFileType() this function will
        // create an SDO document of the required type.
        public static Document createCoreAPIDocOfType(String type)
        {
            Document sourceDocument = null;
            switch (type)
            {
                case "WSDL":
                    sourceDocument = new WSDLDocument();
                    break;

                case "XSD":
                    sourceDocument = new XSDDocument();
                    break;

                case "WS-POLICY":
                    sourceDocument = new PolicyDocument();
                    break;

                case "XML":
                    sourceDocument = new XMLDocument();
                    break;

                case "SCA":
                    sourceDocument = new ModuleDocument();
                    break;

                default:
                    sourceDocument = new GenericDocument();
                    break;
            }

            return sourceDocument;
        }

        // This function adds the classification conditions specified to the XPath specified.
        // First all existing classification conditions are removed from the XPath expression and then
        // the array of classifications are added with the phrase specified by matchAll and includeChildren:
        //      matchAll false, includeChildren false: exactlyClassifiedByAnyOf
        //      matchAll  true, includeChildren  true: classifiedByAllOf
        //      matchAll false, includeChildren false: exactlyClassifiedByAnyOf
        //      matchAll  true, includeChildren  true: classifiedByAllOf                
        public static String filterByClassification(String xPath, String[] classifications, bool matchAll, bool includeChildren)
        {
            String []phrases = new string[] { "classifiedByAllOf(", "classifiedByAnyOf(", "exactlyClassifiedByAllOf(", "exactlyClassifiedByAnyOf(" };
            String fullXPath = xPath;
            String andPhrase = "";

            // First remove all classification info from the string
            for (int c=0; c < phrases.Length;c++)
            {
                int index = fullXPath.IndexOf(phrases[c]);

                if (index != -1) {
                    int endIndex = fullXPath.IndexOf(')', index);

                    if (endIndex == -1)
                        endIndex = fullXPath.Length - 1;

                    fullXPath = fullXPath.Remove(index, endIndex - index + 1);
                }
            }

            //Remove any empty []s
            fullXPath = Regex.Replace(fullXPath, "\\[ *\\]", "");

            if (classifications != null && classifications.Length > 0)
            {
                // Check if there is already "and" or "or" at the end of the line
                // if so then don't add another one           
                bool match = Regex.IsMatch(fullXPath, "and +\\]$") || Regex.IsMatch(fullXPath, "and +$")
                            || Regex.IsMatch(fullXPath, "or +\\]$") || Regex.IsMatch(fullXPath, "or +$");
                bool matchNoSpace = Regex.IsMatch(fullXPath, "and\\]$") || Regex.IsMatch(fullXPath, "and$")
                            || Regex.IsMatch(fullXPath, "or\\]$") || Regex.IsMatch(fullXPath, "or$");

                if (match)
                    andPhrase = "";
                else if (matchNoSpace)
                    andPhrase = " ";
                else
                    andPhrase = " and ";

                // Find out if the user has used the [ ] construct
                int startIndex = fullXPath.IndexOf('[');
                int endIndex = fullXPath.IndexOf(']');
                String phrase = null;

                // Add the correct classification condition phrase
                if (includeChildren)
                    if (matchAll)
                        phrase += "classifiedByAllOf(";
                    else
                        phrase += "classifiedByAnyOf(";
                else
                    if (matchAll)
                        phrase += "exactlyClassifiedByAllOf(";
                    else
                        phrase += "exactlyClassifiedByAnyOf(";

                // Add all classifications specified, separate them with commas
                for (int c = 0; c < classifications.Length; c++)
                {
                    phrase += "'" + classifications[c] + "'";

                    if (c < classifications.Length - 1)
                        phrase += ", ";
                }

                phrase += ")";

                if (startIndex > -1)
                {
                    if (endIndex > -1)
                        // the xpath already has [ and ] so insert our classification condition between them
                        fullXPath = fullXPath.Insert(endIndex, andPhrase + phrase);
                    else
                        // the xpath has a [ but no ], so add our condition to the end and also add ]
                        fullXPath += andPhrase + phrase + "]";
                }
                else
                    // xpath had not [ so add out condition to the and and surround it with []
                    fullXPath += "[" + phrase + "]";

            }
            else
            {
                // remove the conjunction if there are no classifications to add.
                // Check if we are just left with "[]", or with a conjunction 
                // with no left or right side
                fullXPath = Regex.Replace(fullXPath, "\\[ *\\]", "");
                fullXPath = Regex.Replace(fullXPath, "\\[ *and *\\]", "");
                fullXPath = Regex.Replace(fullXPath, "\\[ *or *\\]", "");

                fullXPath = Regex.Replace(fullXPath, " *and *\\]$", "]");
                fullXPath = Regex.Replace(fullXPath, " *or *\\]$", "]");
            }

            return fullXPath;
        }

        
        // Load the named query (either GraphQuery or PropertyQuery) from WSRR.
        public static DataGraphType[] loadQuery(WSRRCoreSDOService coreService, String queryName)
        {

            DataGraphType[] result = null;

            // Populate the fields with the info from the loaded query
            try
            {

                // First check for graph queries
                result = Utilities.runGraphQuery(coreService, "/WSRR/GraphQuery[@name='" + queryName + "']", -1, true);

                // If no graph queries found then check for property queries
                if (result.Length == 0)
                    result = Utilities.runGraphQuery(coreService, "/WSRR/PropertyQuery[@name='" + queryName + "']", -1, true);

                return result;
            }
            catch (Exception we)
            {
                Program.comms.HandleException("Could not load query '" + queryName + "': ", we);
                return null;
            }
        }


    }
}
